#include "PalmOSGUI_dbfile.h"

/************************************************/
/* PalmOSGUI_dbfile.c                           */
/* A small package to unify handling of data    */
/* manager databases and VFS .pdb's             */
/************************************************/

void DbFileFromDbRef(DmOpenRef db, DbFile *dbf)
{
	if (!dbf) 
	  return;

	dbf->external = false;
	dbf->u.db = db;
}

void DbFileFromFileRef(FileRef f, DbFile *dbf)
{
	if (!dbf)
	  return;

	dbf->external = true;
	dbf->u.f = f;
}

void DbFileClose(DbFile *dbf)
{
	if (!dbf)
	  return;

	if (dbf->external)
		VFSFileClose(dbf->u.f);
	else
		DmCloseDatabase(dbf->u.db);
}

Err DbFileInfo(DbFile *dbf, UInt32 *type, UInt32 *app, UInt16 *n)
{
	Err e = errNone;
	UInt16 card;
	LocalID id;

	if (!dbf)
	  return dmErrInvalidParam;

	if (dbf->external)
	{
		e = VFSFileDBInfo(dbf->u.f, NULL, NULL, NULL, NULL, NULL,
			NULL, NULL, NULL, NULL, type, app, n);
	} 
	else
	{
		if(n)
			*n = DmNumRecords(dbf->u.db);
		if (type || app)
		{
			e = DmOpenDatabaseInfo(dbf->u.db, &id, NULL, NULL, &card, NULL);
			if(e == errNone)
			{
  			e = DmDatabaseInfo(card, id, NULL, NULL, NULL, NULL, NULL,
  				NULL, NULL, NULL, NULL, type, app);
			}
		}
	}
	return e;
}

Err DbFileGetRecord(DbFile *dbf, UInt16 idx, MemHandle *h)
{
	Err e = errNone;

	if (!dbf)
	  return dmErrInvalidParam;

	if (dbf->external)
	{
		e = VFSFileDBGetRecord(dbf->u.f, idx, h, NULL, NULL);
	}
	else
	{
		*h = DmGetRecord(dbf->u.db, idx);
		if (!*h) 
		  e = DmGetLastErr();
	}
	return e;
}

Err DbFileReleaseRecord(DbFile *dbf, UInt16 idx, MemHandle h, Boolean b)
{
	if (!dbf) 
	  return errNone;

	if (dbf->external)
		MemHandleFree(h);
	else
		DmReleaseRecord(dbf->u.db, idx, b);

	return errNone;
}
