/*
 *  Prefs.cpp - Global preferences
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#include "sysdeps.h"

#include "Prefs.h"
#include "Display.h"
#include "C64.h"
#include "main.h"

/*
 *  Constructor: Set up preferences with defaults
 */

Prefs::Prefs()
{
	NormalCycles = 63;
	BadLineCycles = 23;
	CIACycles = 63;
	FloppyCycles = 64;
	SkipFrames = 2;

	DriveType = DRVTYPE_DIR;
	DrivePath[0]='\0';

	SIDType = SIDTYPE_DIGITAL;
	DisplayType = DISPTYPE_WINDOW;

	SpritesOn = true;
	SpriteCollisions = true;
	Joystick1On = true;
	Joystick2On = false;
	JoystickSwap = false;
	LimitSpeed = true;
	FastReset = false;
	CIAIRQHack = false;
	Emul1541Proc = false;
	SIDFilters = false;
	AdaptiveFrameSkip = true;
	BordersOn = true;
  SingleCycleEmulation = false;
  SIDOn = false;
}


Prefs::Prefs(const Prefs &p)
{
	memcpy(this, &p, sizeof(Prefs));
}
const Prefs& Prefs::operator=(const Prefs& p)
{
	memcpy(this, &p, sizeof(Prefs));
	return *this;
}


/*
 *  Check if two Prefs structures are equal
 */

bool Prefs::operator==(const Prefs &rhs) const
{
	return (1
		&& NormalCycles == rhs.NormalCycles
		&& BadLineCycles == rhs.BadLineCycles
		&& CIACycles == rhs.CIACycles
		&& FloppyCycles == rhs.FloppyCycles
		&& SkipFrames == rhs.SkipFrames
		&& DriveType == rhs.DriveType
		&& strcmp(DrivePath, rhs.DrivePath) == 0
		&& SIDType == rhs.SIDType
		&& DisplayType == rhs.DisplayType
		&& SpritesOn == rhs.SpritesOn
		&& SpriteCollisions == rhs.SpriteCollisions
		&& Joystick1On == rhs.Joystick1On
		&& Joystick2On == rhs.Joystick2On
		&& JoystickSwap == rhs.JoystickSwap
		&& LimitSpeed == rhs.LimitSpeed
		&& FastReset == rhs.FastReset
		&& CIAIRQHack == rhs.CIAIRQHack
		&& Emul1541Proc == rhs.Emul1541Proc
		&& SIDFilters == rhs.SIDFilters
		&& SingleCycleEmulation == rhs.SingleCycleEmulation
		&& SIDOn == rhs.SIDOn
	);
}

bool Prefs::operator!=(const Prefs &rhs) const
{
	return !operator==(rhs);
}


/*
 *  Check preferences for validity and correct if necessary
 */

void Prefs::Check(void)
{
	if (SkipFrames <= 0) SkipFrames = 1;

	if (SIDType < SIDTYPE_NONE || SIDType > SIDTYPE_SIDCARD)
		SIDType = SIDTYPE_NONE;

	if (DisplayType < DISPTYPE_WINDOW || DisplayType > DISPTYPE_SCREEN)
		DisplayType = DISPTYPE_WINDOW;

	if (DriveType < DRVTYPE_DIR || DriveType > DRVTYPE_T64)
		DriveType = DRVTYPE_DIR;
}


/*
 *  Load preferences from file
 */

void Prefs::SetState(PrefsState *state)
{
	NormalCycles = state->NormalCycles;
	BadLineCycles = state->BadLineCycles;
	CIACycles = state->CIACycles;
	FloppyCycles = state->FloppyCycles;
	SkipFrames = state->SkipFrames;
	DriveType = state->DriveType;
	strncpy(DrivePath, state->DrivePath, 32);
	DrivePath[32]='\0';
	SIDType = state->SIDType;
	SpritesOn = state->SpritesOn!=0;
	SpriteCollisions = state->SpriteCollisions!=0;
	Joystick1On = state->Joystick1On!=0;
	Joystick2On = state->Joystick2On!=0;
	JoystickSwap = state->JoystickSwap!=0;
	LimitSpeed = state->LimitSpeed!=0;
	FastReset = state->FastReset!=0;
	CIAIRQHack = state->CIAIRQHack!=0;
	Emul1541Proc = state->Emul1541Proc!=0;
	SIDFilters = state->SIDFilters!=0;
	BordersOn = state->BordersOn!=0;
	SingleCycleEmulation = state->SingleCycleEmulation!=0;
  SIDOn = state->SIDOn!=0;
  
	Check();
}


/*
 *  Save preferences to file
 *  true: success, false: error
 */

bool Prefs::GetState(PrefsState *state)
{
	Check();
	memset(state, 0x0, sizeof(struct PrefsState));
	state->NormalCycles = NormalCycles;
	state->BadLineCycles = BadLineCycles;
	state->CIACycles = CIACycles;
	state->FloppyCycles = FloppyCycles;
	state->AdaptiveFrameSkip = AdaptiveFrameSkip;
	state->SkipFrames = SkipFrames;
	state->DriveType = DriveType;
	strncpy(state->DrivePath, DrivePath, 31);
	state->SIDType = SIDType;
	state->SpritesOn = SpritesOn? 1:0;
	state->SpriteCollisions = SpriteCollisions? 1:0;
	state->Joystick1On = Joystick1On? 1:0;
	state->Joystick2On = Joystick2On? 1:0;
	state->JoystickSwap = JoystickSwap? 1:0;
	state->LimitSpeed = LimitSpeed? 1:0;
	state->FastReset = FastReset? 1:0;
	state->CIAIRQHack = CIAIRQHack? 1:0;
	state->Emul1541Proc = Emul1541Proc? 1:0;
	state->SIDFilters = SIDFilters? 1:0;
	state->BordersOn = BordersOn? 1:0;
	state->SingleCycleEmulation = SingleCycleEmulation? 1:0;
	state->SIDOn = SIDOn? 1:0;
	
	return true;
}
