/*
 *  Prefs.h - Global preferences
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#ifndef _PREFS_H
#define _PREFS_H

#include "PrefsState_PalmOS.h"

// SID types
enum {
	SIDTYPE_NONE,		// SID emulation off
	SIDTYPE_DIGITAL,	// Digital SID emulation
	SIDTYPE_SIDCARD		// SID card
};


// Display types
enum {
	DISPTYPE_WINDOW,	// BWindow
	DISPTYPE_SCREEN		// BWindowScreen
};

// Preferences data
class Prefs {
public:
	Prefs();
	bool ShowEditor(bool startup, char *prefs_name);
	void Check(void);
	Prefs(const Prefs &p);
	void SetState(PrefsState *p);
	bool GetState(PrefsState *p);
	const Prefs& operator=(const Prefs& p);

	bool operator==(const Prefs &rhs) const;
	bool operator!=(const Prefs &rhs) const;

	int NormalCycles;		// Available CPU cycles in normal raster lines
	int BadLineCycles;		// Available CPU cycles in Bad Lines
	int CIACycles;			// CIA timer ticks per raster line
	int FloppyCycles;		// Available 1541 CPU cycles per line
	int SkipFrames;			// Draw every n-th frame

	int DriveType;		// Type of drive 8..11

	char DrivePath[256];	// Path for drive 8..11
  
	int SIDType;			// SID emulation type
	int DisplayType;		// Display type (BeOS)

	bool SpritesOn;			// Sprite display is on
	bool SpriteCollisions;	// Sprite collision detection is on
	bool Joystick1On;		// Joystick connected to port 1 of host
	bool Joystick2On;		// Joystick connected to port 2 of host
	bool JoystickSwap;		// Swap joysticks 1<->2
	bool LimitSpeed;		// Limit speed to 100%
	bool FastReset;			// Skip RAM test on reset
	bool CIAIRQHack;		// Write to CIA ICR clears IRQ
	bool Emul1541Proc;		// Enable processor-level 1541 emulation
	bool SIDFilters;		// Emulate SID filters

	bool AdaptiveFrameSkip;
	bool BordersOn;
	bool SingleCycleEmulation;
	bool SIDOn;
};


#endif
